@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2019 Meltytech, LLC
+ * Copyright (c) 2013-2020 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include "timelinecommands.h"
 #include "mltcontroller.h"
 #include "shotcut_mlt_properties.h"
+#include "settings.h"
 #include <Logger.h>
 #include <QMetaObject>
 
@@ -58,6 +59,7 @@ InsertCommand::InsertCommand(MultitrackModel &model, int trackIndex,
     , m_xml(xml)
     , m_undoHelper(m_model)
     , m_seek(seek)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
 {
     setText(QObject::tr("Insert into track"));
 }
@@ -74,10 +76,10 @@ void InsertCommand::redo()
             QScopedPointer<Mlt::ClipInfo> info(playlist.clip_info(i));
             clip = Mlt::Producer(info->producer);
             clip.set_in_and_out(info->frame_in, info->frame_out);
-            m_model.insertClip(m_trackIndex, clip, m_position, false);
+            m_model.insertClip(m_trackIndex, clip, m_position, m_rippleAllTracks, false);
         }
     } else {
-        m_model.insertClip(m_trackIndex, clip, m_position, m_seek);
+        m_model.insertClip(m_trackIndex, clip, m_position, m_rippleAllTracks, m_seek);
     }
     m_undoHelper.recordAfterState();
 }
@@ -129,15 +131,15 @@ void OverwriteCommand::undo()
 }
 
 LiftCommand::LiftCommand(MultitrackModel &model, int trackIndex,
-    int clipIndex, const QString &xml, QUndoCommand *parent)
+    int clipIndex, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
     , m_trackIndex(trackIndex)
     , m_clipIndex(clipIndex)
-    , m_xml(xml)
     , m_undoHelper(m_model)
 {
     setText(QObject::tr("Lift from track"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
 }
 
 void LiftCommand::redo()
@@ -155,22 +157,23 @@ void LiftCommand::undo()
 }
 
 RemoveCommand::RemoveCommand(MultitrackModel &model, int trackIndex,
-    int clipIndex, const QString &xml, QUndoCommand *parent)
+    int clipIndex, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
     , m_trackIndex(trackIndex)
     , m_clipIndex(clipIndex)
-    , m_xml(xml)
     , m_undoHelper(m_model)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
 {
     setText(QObject::tr("Remove from track"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
 }
 
 void RemoveCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex;
     m_undoHelper.recordBeforeState();
-    m_model.removeClip(m_trackIndex, m_clipIndex);
+    m_model.removeClip(m_trackIndex, m_clipIndex, m_rippleAllTracks);
     m_undoHelper.recordAfterState();
 }
 
@@ -315,33 +318,111 @@ void LockTrackCommand::undo()
     m_model.setTrackLock(m_trackIndex, m_oldValue);
 }
 
-MoveClipCommand::MoveClipCommand(MultitrackModel &model, int fromTrackIndex, int toTrackIndex, int clipIndex, int position, bool ripple, QUndoCommand *parent)
+MoveClipCommand::MoveClipCommand(MultitrackModel &model, int trackDelta, bool ripple, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
-    , m_fromTrackIndex(fromTrackIndex)
-    , m_toTrackIndex(toTrackIndex)
-    , m_fromClipIndex(clipIndex)
-    , m_fromStart(model.data(
-        m_model.index(clipIndex, 0, m_model.index(fromTrackIndex)),
-            MultitrackModel::StartRole).toInt())
-    , m_toStart(position)
+    , m_trackDelta(trackDelta)
     , m_ripple(ripple)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
     , m_undoHelper(m_model)
+    , m_redo(false)
+    , m_start(-1)
 {
-    setText(QObject::tr("Move clip"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
+    m_undoHelper.recordBeforeState();
 }
 
 void MoveClipCommand::redo()
 {
-    LOG_DEBUG() << "fromTrack" << m_fromTrackIndex << "toTrack" << m_toTrackIndex;
-    m_undoHelper.recordBeforeState();
-    m_model.moveClip(m_fromTrackIndex, m_toTrackIndex, m_fromClipIndex, m_toStart, m_ripple);
-    m_undoHelper.recordAfterState();
+    LOG_DEBUG() << "track delta" << m_trackDelta;
+    int trackIndex, clipIndex;
+    QMultiMap<int, Mlt::Producer> newSelection;
+
+    if (!m_redo) {
+        if (m_selection.size() > 1)
+            setText(QObject::tr("Move %1 timelime clips").arg(m_selection.size()));
+        else
+            setText(QObject::tr("Move timelime clip"));
+    }
+    if (m_ripple && !m_trackDelta && m_selection.size() == 1) {
+        if (m_start == -1) {
+            QScopedPointer<Mlt::ClipInfo> info(m_model.findClipByUuid(MLT.uuid(m_selection.first()), trackIndex, clipIndex));
+            auto newStart = info->cut->get_int(kPlaylistStartProperty);
+            auto mlt_index = m_model.trackList().at(trackIndex).mlt_index;
+            QScopedPointer<Mlt::Producer> track(m_model.tractor()->track(mlt_index));
+            if (track) {
+                Mlt::Playlist playlist(*track);
+                auto targetIndex = playlist.get_clip_index_at(newStart);
+                auto length = playlist.clip_length(clipIndex);
+                auto targetIndexEnd = playlist.get_clip_index_at(newStart + length - 1);
+                if (targetIndex >= clipIndex || // pushing clips on same track
+                        // pulling clips on same track
+                        ((playlist.is_blank_at(newStart) || targetIndex == clipIndex) &&
+                         (playlist.is_blank_at(newStart + length - 1) || targetIndexEnd == clipIndex))) {
+                    m_start = newStart;
+                    m_trackIndex = trackIndex;
+                    m_clipIndex = clipIndex;
+                }
+            }
+        }
+        if (m_start >= 0) {
+            // Use old behavior to push or pull clips on the same track.
+            m_model.moveClip(m_trackIndex, m_trackIndex, m_clipIndex, m_start, m_ripple, m_rippleAllTracks);
+            m_undoHelper.recordAfterState();
+            m_redo = true;
+            return;
+        }
+    }
+    // First, remove each clip.
+    for (auto& clip : m_selection) {
+        if (!m_redo) {
+            // On the initial pass, remove clips while recording info about them.
+            QScopedPointer<Mlt::ClipInfo> info(m_model.findClipByUuid(MLT.uuid(clip), trackIndex, clipIndex));
+            if (info->producer && info->producer->is_valid()) {
+                info->producer->set(kPlaylistIndexProperty, qBound(0, trackIndex + m_trackDelta, m_model.trackList().size() - 1));
+                info->producer->pass_property(*info->cut, kPlaylistStartProperty);
+                info->producer->set("_shotcut:trackIndex", trackIndex);
+                info->producer->set("_shotcut:clipIndex", clipIndex);
+                info->producer->set("_shotcut:in", info->frame_in);
+                info->producer->set("_shotcut:out", info->frame_out);
+                newSelection.insert(info->cut->get_int(kPlaylistStartProperty), info->producer);
+            }
+        } else {
+            // On redo, use the recorded indices to remove them.
+            trackIndex = clip.get_int("_shotcut:trackIndex");
+            clipIndex = clip.get_int("_shotcut:clipIndex");
+        }
+        if (m_ripple)
+            m_model.removeClip(trackIndex, clipIndex, m_rippleAllTracks);
+        else
+            m_model.liftClip(trackIndex, clipIndex);
+    }
+    if (!m_redo) {
+        m_selection = newSelection;
+    }
+    // Next, add the save clips to their new locations.
+    for (auto& clip : m_selection) {
+        auto toTrack = clip.get_int(kPlaylistIndexProperty);
+        auto start = clip.get_int(kPlaylistStartProperty);
+        clip.set_in_and_out(clip.get_int("_shotcut:in"), clip.get_int("_shotcut:out"));
+        if (start + clip.get_playtime() >= 0) {
+//            LOG_DEBUG() << "adding clip" << clip.get_int("_shotcut:clipIndex") << "to track" << toTrack << "start" << start;
+            if (m_ripple)
+                m_model.insertClip(toTrack, clip, start, m_rippleAllTracks);
+            else
+                m_model.overwrite(toTrack, clip, start, false);
+        }
+    }
+
+    if (!m_redo) {
+        m_redo = true;
+        m_undoHelper.recordAfterState();
+    }
 }
 
 void MoveClipCommand::undo()
 {
-    LOG_DEBUG() << "fromTrack" << m_fromTrackIndex << "toTrack" << m_toTrackIndex;
+    LOG_DEBUG() << "track delta" << m_trackDelta;
     m_undoHelper.undoChanges();
 }
 
@@ -352,6 +433,7 @@ TrimClipInCommand::TrimClipInCommand(MultitrackModel &model, int trackIndex, int
     , m_clipIndex(clipIndex)
     , m_delta(delta)
     , m_ripple(ripple)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
     , m_redo(redo)
 {
     setText(QObject::tr("Trim clip in point"));
@@ -364,7 +446,7 @@ void TrimClipInCommand::redo()
         m_undoHelper.reset(new UndoHelper(m_model));
         if (!m_ripple) m_undoHelper->setHints(UndoHelper::SkipXML);
         m_undoHelper->recordBeforeState();
-        m_model.trimClipIn(m_trackIndex, m_clipIndex, m_delta, m_ripple);
+        m_model.trimClipIn(m_trackIndex, m_clipIndex, m_delta, m_ripple, m_rippleAllTracks);
         m_undoHelper->recordAfterState();
     } else {
         Q_ASSERT(m_undoHelper);
@@ -399,6 +481,7 @@ TrimClipOutCommand::TrimClipOutCommand(MultitrackModel &model, int trackIndex, i
     , m_clipIndex(clipIndex)
     , m_delta(delta)
     , m_ripple(ripple)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
     , m_redo(redo)
 {
     setText(QObject::tr("Trim clip out point"));
@@ -411,7 +494,7 @@ void TrimClipOutCommand::redo()
         if (!m_ripple)
             m_undoHelper->setHints(UndoHelper::SkipXML);
         m_undoHelper->recordBeforeState();
-        m_clipIndex = m_model.trimClipOut(m_trackIndex, m_clipIndex, m_delta, m_ripple);
+        m_clipIndex = m_model.trimClipOut(m_trackIndex, m_clipIndex, m_delta, m_ripple, m_rippleAllTracks);
         m_undoHelper->recordAfterState();
     } else {
         Q_ASSERT(m_undoHelper);
@@ -539,6 +622,7 @@ AddTransitionCommand::AddTransitionCommand(MultitrackModel &model, int trackInde
     , m_transitionIndex(-1)
     , m_ripple(ripple)
     , m_undoHelper(model)
+    , m_rippleAllTracks(Settings.timelineRippleAllTracks())
 {
     setText(QObject::tr("Add transition"));
 }
@@ -547,7 +631,7 @@ void AddTransitionCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "position" << m_position;
     m_undoHelper.recordBeforeState();
-    m_transitionIndex = m_model.addTransition(m_trackIndex, m_clipIndex, m_position, m_ripple);
+    m_transitionIndex = m_model.addTransition(m_trackIndex, m_clipIndex, m_position, m_ripple, m_rippleAllTracks);
     LOG_DEBUG() << "m_transitionIndex" << m_transitionIndex;
     m_undoHelper.recordAfterState();
 }
@@ -665,7 +749,7 @@ void AddTransitionByTrimInCommand::redo()
     if (m_redo) {
         LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "delta" << m_trimDelta << "duration" << m_duration;
         if (m_trimDelta)
-            m_model.trimClipIn(m_trackIndex, m_clipIndex + 1, m_trimDelta, false);
+            m_model.trimClipIn(m_trackIndex, m_clipIndex + 1, m_trimDelta, false, false);
         m_model.addTransitionByTrimIn(m_trackIndex, m_clipIndex, m_duration);
         if (m_notify && m_clipIndex > 0)
             m_model.notifyClipOut(m_trackIndex, m_clipIndex - 1);
@@ -711,7 +795,7 @@ void RemoveTransitionByTrimInCommand::redo()
         QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
         int n = m_model.data(modelIndex, MultitrackModel::DurationRole).toInt();
         m_model.liftClip(m_trackIndex, m_clipIndex);
-        m_model.trimClipIn(m_trackIndex, m_clipIndex + 1, -n, false);
+        m_model.trimClipIn(m_trackIndex, m_clipIndex + 1, -n, false, false);
         m_model.notifyClipIn(m_trackIndex, m_clipIndex + 1);
     } else {
         m_redo = true;
@@ -746,7 +830,7 @@ void RemoveTransitionByTrimOutCommand::redo()
         QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
         int n = m_model.data(modelIndex, MultitrackModel::DurationRole).toInt();
         m_model.liftClip(m_trackIndex, m_clipIndex);
-        m_model.trimClipOut(m_trackIndex, m_clipIndex - 1, -n, false);
+        m_model.trimClipOut(m_trackIndex, m_clipIndex - 1, -n, false, false);
         m_model.notifyClipOut(m_trackIndex, m_clipIndex - 1);
     } else {
         m_redo = true;
@@ -781,7 +865,7 @@ void AddTransitionByTrimOutCommand::redo()
     if (m_redo) {
         LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "delta" << m_trimDelta << "duration" << m_duration;
         if (m_trimDelta)
-            m_model.trimClipOut(m_trackIndex, m_clipIndex, m_trimDelta, false);
+            m_model.trimClipOut(m_trackIndex, m_clipIndex, m_trimDelta, false, false);
         m_model.addTransitionByTrimOut(m_trackIndex, m_clipIndex, m_duration);
         if (m_notify)
             m_model.notifyClipIn(m_trackIndex, m_clipIndex + 2);
@@ -870,11 +954,19 @@ RemoveTrackCommand::RemoveTrackCommand(MultitrackModel& model, int trackIndex, Q
     else if (m_trackType == VideoTrackType)
         setText(QObject::tr("Remove video track"));
 
-    // Save track name.
+    // Get the track as MLT playlist.
     int mlt_index = m_model.trackList().at(m_trackIndex).mlt_index;
     QScopedPointer<Mlt::Producer> producer(m_model.tractor()->multitrack()->track(mlt_index));
-    if (producer && producer->is_valid())
+    if (producer && producer->is_valid()) {
+        // Save track name.
         m_trackName = QString::fromUtf8(producer->get(kTrackNameProperty));
+        // Save the track filters.
+        if (producer->filter_count() > 0) {
+            m_filtersProducer.reset(new Mlt::Producer(MLT.profile(), "color"));
+            if (m_filtersProducer->is_valid())
+                MLT.copyFilters(*producer, *m_filtersProducer);
+        }
+    }
 }
 
 void RemoveTrackCommand::redo()
@@ -902,11 +994,10 @@ void RemoveTrackCommand::undo()
     int mlt_index = m_model.trackList().at(m_trackIndex).mlt_index;
     QScopedPointer<Mlt::Producer> producer(m_model.tractor()->multitrack()->track(mlt_index));
     Mlt::Playlist playlist(*producer);
-    int n = playlist.filter_count();
-    for (int i = 0; i < n; ++i) {
-        QScopedPointer<Mlt::Filter> filter(playlist.filter(i));
-        if (filter && filter->is_valid())
-            producer->attach(*filter);
+    if (playlist.is_valid() && m_filtersProducer &&  m_filtersProducer->is_valid()) {
+        MLT.copyFilters(*m_filtersProducer, playlist);
+        QModelIndex modelIndex = m_model.index(m_trackIndex);
+        emit m_model.dataChanged(modelIndex, modelIndex, QVector<int>() << MultitrackModel::IsFilteredRole);
     }
 }
 
@@ -1054,7 +1145,7 @@ void DetachAudioCommand::redo()
         if (m_targetTrackIndex > -1) {
             // Add the clip to the new audio track.
             m_undoHelper.recordBeforeState();
-            m_model.overwrite(m_targetTrackIndex, clip, m_position);
+            m_model.overwrite(m_targetTrackIndex, clip, m_position, false);
             m_undoHelper.recordAfterState();
         }
     }
